@@ -14,38 +14,48 @@ use Carp 'confess';
 
 # singleton for storing settings
 my $SETTINGS = {
-	# default mime_types
-	mime_types => Dancer::FileUtils->mime_types,
+
+    # user defined mime types
+    mime_types => {},
 };
-sub settings { $SETTINGS }
+sub settings {$SETTINGS}
 
 my $setters = {
     logger => sub {
         my ($setting, $value) = @_;
-        Dancer::Logger->init($value);
+        Dancer::Logger->init($value, settings());
     },
     session => sub {
         my ($setting, $value) = @_;
-        Dancer::Session->init($value);
+        Dancer::Session->init($value, settings());
     },
     template => sub {
         my ($setting, $value) = @_;
-        Dancer::Template->init($value);
+        Dancer::Template->init($value, settings());
+    },
+    route_cache => sub {
+        my ($setting, $value) = @_;
+        Dancer::Route::Cache->reset();
+    },
+    serializer => sub {
+        my ($setting, $value) = @_;
+        require Dancer::Serializer;
+        Dancer::Serializer->init($value);
     },
 };
 
 # public accessor for get/set
 sub setting {
     my ($setting, $value) = @_;
-    
+
     # run the hook if setter
-    $setters->{$setting}->(@_) 
-        if (@_ == 2) && defined $setters->{$setting};
+    $setters->{$setting}->(@_)
+      if (@_ == 2) && defined $setters->{$setting};
 
     # setter/getter
-    (@_ == 2) 
-        ? $SETTINGS->{$setting} = $value
-        : $SETTINGS->{$setting} ;
+    (@_ == 2)
+      ? $SETTINGS->{$setting} = $value
+      : $SETTINGS->{$setting};
 }
 
 sub mime_types {
@@ -53,19 +63,26 @@ sub mime_types {
     $SETTINGS->{mime_types} ||= {};
     return $SETTINGS->{mime_types} if @_ == 0;
 
-    return (@_ == 2) 
-        ? $SETTINGS->{mime_types}{$ext} = $content_type
-        : $SETTINGS->{mime_types}{$ext};
+    return (@_ == 2)
+      ? $SETTINGS->{mime_types}{$ext} = $content_type
+      : $SETTINGS->{mime_types}{$ext};
 }
 
-sub conffile { path(setting('appdir'), 'config.yml') }
+sub conffile { path(setting('confdir') || setting('appdir'), 'config.yml') }
 
 sub environment_file {
     my $env = setting('environment');
     return path(setting('appdir'), 'environments', "$env.yml");
 }
 
-sub load { 
+sub init_confdir {
+    return setting('confdir') if setting('confdir');
+    setting confdir => $ENV{DANCER_CONFDIR} || setting('appdir');
+}
+
+sub load {
+    init_confdir();
+
     # look for the conffile
     return 1 unless -f conffile;
 
@@ -79,33 +96,38 @@ sub load {
     my $env = environment_file;
     load_settings_from_yaml($env) if -f $env;
 
+    foreach my $key (grep { $setters->{$_} } keys %$SETTINGS) {
+        $setters->{$key}->($key, $SETTINGS->{$key});
+    }
+
     return 1;
 }
 
 sub load_settings_from_yaml {
     my ($file) = @_;
 
-    my $config = YAML::LoadFile($file) or 
-        confess "Unable to parse the configuration file: $file";
+    my $config = YAML::LoadFile($file)
+      or confess "Unable to parse the configuration file: $file";
 
-    foreach my $key (keys %$config) {
-        # set values for new settings
-        setting($key => $config->{$key});
-    }
+    @{$SETTINGS}{keys %$config} = values %$config;
     return scalar(keys %$config);
 }
 
+
 sub load_default_settings {
-    $SETTINGS->{server}       ||= '127.0.0.1';
-    $SETTINGS->{port}         ||= '3000';
-    $SETTINGS->{content_type} ||= 'text/html';
-    $SETTINGS->{charset}      ||= 'UTF-8';
-    $SETTINGS->{access_log}   ||= 1;
-    $SETTINGS->{daemon}       ||= 0;
-    $SETTINGS->{environment}  ||= 'development';
-    $SETTINGS->{apphandler}   ||= 'standalone';
-    $SETTINGS->{warnings}     ||= 0;
-    $SETTINGS->{auto_reload}  ||= 0;
+    $SETTINGS->{server}       ||= $ENV{DANCER_SERVER} || '0.0.0.0';
+    $SETTINGS->{port}         ||= $ENV{DANCER_PORT} || '3000';
+    $SETTINGS->{content_type} ||= $ENV{DANCER_CONTENT_TYPE} || 'text/html';
+    $SETTINGS->{charset}      ||= $ENV{DANCER_CHARSET} || 'UTF-8';
+    $SETTINGS->{access_log}   ||= $ENV{DANCER_ACCESS_LOG} || 1;
+    $SETTINGS->{daemon}       ||= $ENV{DANCER_DAEMON} || 0;
+    $SETTINGS->{apphandler}   ||= $ENV{DANCER_APPHANDLER} || 'standalone';
+    $SETTINGS->{warnings}     ||= $ENV{DANCER_WARNINGS} || 0;
+    $SETTINGS->{auto_reload}  ||= $ENV{DANCER_AUTO_RELOAD} || 0;
+    $SETTINGS->{environment}
+      ||= $ENV{DANCER_ENVIRONMENT}
+      || $ENV{PLACK_ENV}
+      || 'development';
 
     setting template => 'simple';
 }
@@ -113,6 +135,7 @@ load_default_settings();
 
 1;
 __END__
+
 =pod
 
 =head1 NAME
@@ -171,12 +194,12 @@ This is the path of the public directory, where static files are stored. Any
 existing file in that directory will be served as a static file, before
 mathcing any route.
 
-By default, it points to APPDIR/public where APPDIR is the directory that 
+By default, it points to APPDIR/public where APPDIR is the directory that
 contains your Dancer script.
 
 =head2 layout (string)
 
-name of the layout to use when rendering view. Dancer will look for 
+name of the layout to use when rendering view. Dancer will look for
 a matching template in the directory $appdir/views/layout.
 
 =head2 warnings (boolean)
@@ -206,9 +229,12 @@ $public/$error_code.html if it exists.
 
 =head2 auto_reload (boolean)
 
+Requires L<Module::Refresh>.
+
 If set to true, Dancer will reload the route handlers whenever the file where
 they are defined is changed. This is very useful in development environment but
-should not be enabled in production.
+B<should not be enabled in production>. Enabling this flag in production yields
+a major negative effect on performance because of L<Module::Refresh>.
 
 When this flag is set, you don't have to restart your webserver whenever you
 make a change in a route handler.
